﻿Imports java.util
Imports java.net
Imports com.ihr.xbrl.om
Imports com.ihr.xbrl.om.xdt
Imports com.ihr.xbrl.om.instance
Imports com.ihr.xbrl.om.taxonomy
Imports javax.xml.namespace
Imports net.sf.saxon.value
Imports java.lang

Module Module1

    Sub Main()

        ' Configure the XBRL processor according to needs. In this case,
        ' the taxonomy contains dimensions so let's add the XDTProcessor
        ' to the list of processors in the validation sequence.
        Dim props As Properties = New Properties()

        Const processorsSequence = XBRLCoreProcessor.PROCESSORNAME + "," + XDTProcessor.PROCESSORNAME

        props.setProperty(DTSContainer.PROCESSORS_SEQUENCE, processorsSequence)
        props.setProperty(DTSContainer.LOG_LEVEL, "ALL")

        ' for more information about configuration parameters visit the DTSContainer documentation at
        ' http://www.reportingstandard.com/apidoc/com/ihr/xbrl/om/DTSContainer.html

        ' create a new empty container
        Dim dts As DTSContainer = DTSContainer.newEmptyContainer(props)

        ' 1. Open the DTS (Taxonomy) from the folowing two entry points. Note by default the taxonomy catalog
        ' will be used so no internet access is needed and no files will be opened from the Internet
        ' Note: this is a very large taxonomy and we are using 5 entry points and a couple of hundred files from here are "discovered"
        Dim lbPres As URI = URI.create("http://xbrl.ifrs.org/taxonomy/2011-03-25/full_ifrs/ias_1_2011-03-25/pre_ias_1_2011-03-25_role-110000.xml")
        Dim lbLabels As URI = URI.create("http://xbrl.ifrs.org/taxonomy/2011-03-25/labels/lab_ifrs-en_2011-03-25.xml")
        Dim lbPre As URI = URI.create("http://xbrl.ifrs.org/taxonomy/2011-03-25/full_ifrs/ias_1_2011-03-25/pre_ias_1_2011-03-25_role-220000.xml")
        Dim lbPre2 As URI = URI.create("http://xbrl.ifrs.org/taxonomy/2011-03-25/full_ifrs/ias_1_2011-03-25/pre_ias_1_2011-03-25_role-110000.xml")
        Dim lbCal As URI = URI.create("http://xbrl.ifrs.org/taxonomy/2011-03-25/full_ifrs/ias_1_2011-03-25/cal_ias_1_2011-03-25_role-220000.xml")
        dts.load(New URI() {lbPres, lbLabels, lbPre, lbPre2, lbCal})

        '// we will be using the IFRS namespace so delcare it
        Dim IFRS As String = "http://xbrl.ifrs.org/taxonomy/2011-03-25/ifrs"

        '// 2. Create a new Empty XBRL Report
        Dim report As XBRLInstance = New XBRLInstance(dts)

        '// 3. Add fact values to the XBRL report

        '// Create two XBRL context 
        Dim company As XBRLEntity = New XBRLEntity(dts, "http://www.xbrl.org/scheme", "Sample Company", Nothing)
        Dim periodD As XBRLPeriod = New XBRLPeriod(dts, "2010-01-01", "2010-12-31")
        Dim ctxD As XBRLContext = New XBRLContext(dts, company, periodD, Nothing)

        Dim periodI As XBRLPeriod = New XBRLPeriod(dts, "2010-12-31")
        Dim ctxI As XBRLContext = New XBRLContext(dts, company, periodI, Nothing)

        '// add the first item
        Dim NameOfReportingEntityOrOtherMeansOfIdentificationItem As XBRLItem = dts.getConcept(New QName(IFRS, "NameOfReportingEntityOrOtherMeansOfIdentification"))
        Dim f1 As XBRLFactItem = New XBRLFactNonNumeric(report.getInstanceRootNode(), ctxD, NameOfReportingEntityOrOtherMeansOfIdentificationItem)
        f1.setValue(New StringValue("Sample Company"))

        '// add other items

        Dim PeriodCoveredByFinancialStatementsItem As XBRLItem = dts.getConcept(New QName(IFRS, "PeriodCoveredByFinancialStatements"))
        Dim f2 As XBRLFactItem = New XBRLFactNonNumeric(report.getInstanceRootNode(), ctxD, PeriodCoveredByFinancialStatementsItem)
        f2.setValue(New StringValue("Year 2010"))

        Dim DescriptionOfNatureOfFinancialStatementsItem As XBRLItem = dts.getConcept(New QName(IFRS, "DescriptionOfNatureOfFinancialStatements"))
        Dim f3 As XBRLFactItem = New XBRLFactNonNumeric(report.getInstanceRootNode(), ctxD, DescriptionOfNatureOfFinancialStatementsItem)
        f3.setValue(New StringValue("Balance Sheet"))

        Dim numeratorUnits As Vector = New Vector()
        numeratorUnits.add(New QName("http://www.xbrl.org/2003/iso4217", "EUR", "iso4217"))
        Dim unit As XBRLUnit = New XBRLUnit(dts, numeratorUnits, Nothing)

        Dim PropertyPlantAndEquipmentItem As XBRLItem = dts.getConcept(New QName(IFRS, "PropertyPlantAndEquipment"))
        Dim f4 As XBRLFactNumeric = New XBRLFactNumeric(report.getInstanceRootNode(), ctxI, PropertyPlantAndEquipmentItem, unit, New java.lang.Integer(12), XBRLNumber.__Fields.INF, XBRLNumber.ExactitudeMode.DECIMALS, Nothing)

        Dim InvestmentPropertyItem As XBRLItem = dts.getConcept(New QName(IFRS, "InvestmentProperty"))
        Dim f5 As XBRLFactNumeric = New XBRLFactNumeric(report.getInstanceRootNode(), ctxI, InvestmentPropertyItem, unit, New java.lang.Integer(34), XBRLNumber.__Fields.INF, XBRLNumber.ExactitudeMode.DECIMALS, Nothing)

        Dim GoodwillItem As XBRLItem = dts.getConcept(New QName(IFRS, "Goodwill"))
        Dim f6 As XBRLFactNumeric = New XBRLFactNumeric(report.getInstanceRootNode(), ctxI, GoodwillItem, unit, New java.lang.Integer(56), XBRLNumber.__Fields.INF, XBRLNumber.ExactitudeMode.DECIMALS, Nothing)

        '// 4. Validate the new XBRL Report against the specifications
        Dim valReport As XBRLValidationReport = dts.validate()

        '// in case of errors just raise them.
        valReport.throwExceptions()
        valReport.inconsistenciesAsExceptions()

        '// 5. Save the XBRL report to disk
        Dim docName As URI = New URI("sampleReport.xbrl")
        report.setURI(docName)
        report.save()

    End Sub

End Module
